@@ -2,16 +2,23 @@ from robot.output import LOGGER
 
 from base64 import b64encode
 from functools import wraps
+from urlparse import urlparse
 
 import livetest
 import json
 import jsonpointer
 
+def load_json(json_string):
+    try:
+        return json.loads(json_string)
+    except ValueError, e:
+        raise ValueError("Could not parse '%s' as JSON: %s" % (json_string, e))
+
 def _with_json(f):
     @wraps(f)
     def wrapper(self, json_string, *args, **kwargs):
         return json.dumps(
-          f(self, json.loads(json_string), *args, **kwargs))
+          f(self, load_json(json_string), *args, **kwargs))
     return wrapper
 
 class HTTP:
@@ -47,6 +54,19 @@ class HTTP:
             raise Exception('No request available, use e.g. GET to create one.')
         return self._response
 
+    def _path_from_url_or_path(self, url_or_path):
+
+        if url_or_path.startswith("/"):
+            return url_or_path
+
+        elif url_or_path.startswith("http"):
+            parsed_url = urlparse(url_or_path)
+            self.set_http_host(parsed_url.netloc)
+            return parsed_url.path
+
+        raise Exception('"%s" needs to be in form of "/path" or "http://host/path"'
+                % url_or_path)
+
     # setup
 
     def set_http_host(self, host):
@@ -60,13 +80,27 @@ class HTTP:
 
     # request
 
+    def http_request(self, verb, url):
+        """
+        Issues a HTTP request with an uncommon HTTP Verb.
+
+        `verb` is the HTTP Verb to use, e.g. "PROPFIND", "PATCH", "OPTIONS"
+        `url` is the URL relative to the server root, e.g. '/_utils/config.html'
+        """
+        path = self._path_from_url_or_path(url)
+
+        self._response = self._app.request(path, {}, self._request_headers,
+                method=verb.upper(),)
+        self._reset()
+
     def HEAD(self, url):
         """
         Issues a HTTP HEAD request.
 
         `url` is the URL relative to the server root, e.g. '/_utils/config.html'
         """
-        self._response = self.app.head(url, {}, self._request_headers)
+        path = self._path_from_url_or_path(url)
+        self._response = self.app.head(path, {}, self._request_headers)
         self._reset()
 
     def GET(self, url):
@@ -75,7 +109,8 @@ class HTTP:
 
         `url` is the URL relative to the server root, e.g. '/_utils/config.html'
         """
-        self._response = self.app.get(url, {}, self._request_headers)
+        path = self._path_from_url_or_path(url)
+        self._response = self.app.get(path, {}, self._request_headers)
         self._reset()
 
     def POST(self, url):
@@ -84,10 +119,11 @@ class HTTP:
 
         `url` is the URL relative to the server root, e.g. '/_utils/config.html'
         """
+        path = self._path_from_url_or_path(url)
         kwargs = {}
         if 'Content-Type' in self._request_headers:
             kwargs['content_type'] = self._request_headers['Content-Type']
-        self._response = self.app.post(url, self._request_body or {}, self._request_headers, **kwargs)
+        self._response = self.app.post(path, self._request_body or {}, self._request_headers, **kwargs)
         self._reset()
 
     def PUT(self, url):
@@ -96,10 +132,11 @@ class HTTP:
 
         `url` is the URL relative to the server root, e.g. '/_utils/config.html'
         """
+        path = self._path_from_url_or_path(url)
         kwargs = {}
         if 'Content-Type' in self._request_headers:
             kwargs['content_type'] = self._request_headers['Content-Type']
-        self._response = self.app.put(url, self._request_body or {}, self._request_headers, **kwargs)
+        self._response = self.app.put(path, self._request_body or {}, self._request_headers, **kwargs)
         self._reset()
 
     def DELETE(self, url):
@@ -108,7 +145,8 @@ class HTTP:
 
         `url` is the URL relative to the server root, e.g. '/_utils/config.html'
         """
-        self._response = self.app.delete(url, {}, self._request_headers)
+        path = self._path_from_url_or_path(url)
+        self._response = self.app.delete(path, {}, self._request_headers)
         self._reset()
 
     def follow_response(self):
@@ -212,6 +250,7 @@ class HTTP:
         `header_name` is the name of the header, e.g. `User-Agent`
         `header_value` is the key of the header, e.g. `RobotFramework HttpLibrary (Mozilla/4.0)`
         """
+        LOGGER.write('Set request header "%s" to "%s"' % (header_name, header_value), 'INFO')
         self._request_headers[header_name] = header_value
 
     def set_basic_auth(self, username, password):
@@ -224,7 +263,9 @@ class HTTP:
 
         `password` is the password to use, e.g. 'open sesame'
         """
-        self.set_request_header("Authorization", "Basic %s" % b64encode("%s:%s" % (username, password)))
+        credentials = "%s:%s" % (username, password)
+        LOGGER.write('Set basic auth to "%s"' % credentials, 'INFO')
+        self.set_request_header("Authorization", "Basic %s" % b64encode(credentials))
 
     # payload
 
@@ -235,8 +276,9 @@ class HTTP:
         Example:
         | Set Request Body           | user=Aladdin&password=open%20sesame |
         | POST                       | /login                              |
-        | `Response Should Succeed`  |                                     |
+        | Response Should Succeed  |                                     |
         """
+        LOGGER.write('Request body set to "%s".' % body, 'INFO')
         self._request_body = body.encode("utf-8")
 
     def get_response_body(self):
@@ -279,7 +321,7 @@ class HTTP:
         Example:
         | Should Be Valid Json | {"foo": "bar"} |
         """
-        json.loads(json_string)
+        load_json(json_string)
 
     @_with_json
     def get_json_value(self, json_string, json_pointer):
@@ -307,6 +349,23 @@ class HTTP:
         assert got == expected_value, \
                'JSON value "%s" does not equal "%s", but should have.' % (got, expected_value)
 
+
+    def json_value_should_not_equal(self, json_string, json_pointer, expected_value):
+        """
+        Fails if the value of the target node of the JSON document
+        `json_string` specified by JSON Pointer `json_pointer` is `expected_value`.
+
+        Example:
+        | Set Test Variable           | ${doc}  | {"foo": {"bar": [1,2,3]}} |             |
+        | Json Value Should Not Equal | ${doc}  | /foo/bar                  | "[1, 2, 3]" |
+        """
+
+        got = self.get_json_value(json_string, json_pointer)
+
+        assert got != expected_value, \
+               'JSON value "%s" does not equal "%s", but should have.' % (got, expected_value)
+
+
     @_with_json
     def set_json_value(self, json_string, json_pointer, json_value):
         """
@@ -317,7 +376,7 @@ class HTTP:
         | ${result}=       | Set Json Value | {"foo": {"bar": [1,2,3]}} | /foo | 12 |
         | Should Be Equal  | ${result}      | {"foo": 12}               |      |    |
         """
-        value = json.loads(json_value)
+        value = load_json(json_value)
         p = jsonpointer.set_pointer(json_string, json_pointer, value)
         return json_string
 
